@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2008 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,12 @@
 #include <time.h>
 #include <gdk/gdk.h>
 
+#include <gconf/gconf-engine.h>
+#include <gconf/gconf-client.h>
+
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnomeui/gnome-bg.h>
+
 #include "gs-prefs.h"        /* for GSSaverMode */
 
 #include "gs-manager.h"
@@ -47,6 +53,9 @@ struct GSManagerPrivate
         GHashTable  *jobs;
 
         GSThemeManager *theme_manager;
+        GConfClient    *client;
+        GnomeBG        *bg;
+        guint           bg_notify_id;
 
         /* Policy */
         glong        lock_timeout;
@@ -906,7 +915,7 @@ gs_manager_class_init (GSManagerClass *klass)
                                                             0,
                                                             G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
-                                         PROP_LOGOUT_TIMEOUT,
+                                         PROP_LOGOUT_COMMAND,
                                          g_param_spec_string ("logout-command",
                                                               NULL,
                                                               NULL,
@@ -932,6 +941,69 @@ gs_manager_class_init (GSManagerClass *klass)
         g_type_class_add_private (klass, sizeof (GSManagerPrivate));
 }
 
+static void
+on_bg_changed (GnomeBG   *bg,
+               GSManager *manager)
+{
+        gs_debug ("background changed");
+}
+
+static void
+gconf_changed_callback (GConfClient *client,
+                        guint        cnxn_id,
+                        GConfEntry  *entry,
+                        GSManager   *manager)
+{
+        gnome_bg_load_from_preferences (manager->priv->bg,
+                                        manager->priv->client);
+}
+
+static void
+watch_bg_preferences (GSManager *manager)
+{
+        g_assert (manager->priv->bg_notify_id == 0);
+
+        gconf_client_add_dir (manager->priv->client,
+                              GNOME_BG_KEY_DIR,
+                              GCONF_CLIENT_PRELOAD_NONE,
+                              NULL);
+        manager->priv->bg_notify_id = gconf_client_notify_add (manager->priv->client,
+                                                               GNOME_BG_KEY_DIR,
+                                                               (GConfClientNotifyFunc)gconf_changed_callback,
+                                                               manager,
+                                                               NULL,
+                                                               NULL);
+}
+
+static GConfClient *
+get_gconf_client (void)
+{
+        GConfClient        *client;
+        GSList             *addresses;
+        GError             *error;
+        GConfEngine        *engine;
+
+        client = NULL;
+        addresses = NULL;
+
+        addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/gconf/gconf.xml.mandatory");
+        addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/gconf/gconf.xml.system");
+        addresses = g_slist_prepend (addresses, "xml:merged:" SYSCONFDIR "/gconf/gconf.xml.defaults");
+        addresses = g_slist_reverse (addresses);
+
+        error = NULL;
+        engine = gconf_engine_get_for_addresses (addresses, &error);
+        if (engine == NULL) {
+                gs_debug ("Unable to get gconf engine for addresses: %s", error->message);
+        } else {
+                client = gconf_client_get_for_engine (engine);
+        }
+
+        g_slist_free (addresses);
+
+        return client;
+}
+
 static void
 gs_manager_init (GSManager *manager)
 {
@@ -940,6 +1012,19 @@ gs_manager_init (GSManager *manager)
         manager->priv->fade = gs_fade_new ();
         manager->priv->grab = gs_grab_new ();
         manager->priv->theme_manager = gs_theme_manager_new ();
+
+        manager->priv->client = get_gconf_client ();
+        if (manager->priv->client != NULL) {
+                manager->priv->bg = gnome_bg_new ();
+
+                g_signal_connect (manager->priv->bg,
+                                  "changed",
+                                  G_CALLBACK (on_bg_changed),
+                                  manager);
+                watch_bg_preferences (manager);
+
+                gnome_bg_load_from_preferences (manager->priv->bg, manager->priv->client);
+        }
 }
 
 static void
@@ -971,6 +1056,21 @@ gs_manager_finalize (GObject *object)
 
         g_return_if_fail (manager->priv != NULL);
 
+        if (manager->priv->bg_notify_id != 0) {
+                gconf_client_remove_dir (manager->priv->client,
+                                         GNOME_BG_KEY_DIR,
+                                         NULL);
+                gconf_client_notify_remove (manager->priv->client,
+                                            manager->priv->bg_notify_id);
+                manager->priv->bg_notify_id = 0;
+        }
+        if (manager->priv->bg != NULL) {
+                g_object_unref (manager->priv->bg);
+        }
+        if (manager->priv->client != NULL) {
+                g_object_unref (manager->priv->client);
+        }
+
         free_themes (manager);
         g_free (manager->priv->logout_command);
         g_free (manager->priv->keyboard_command);
@@ -1216,12 +1316,41 @@ window_unmap_cb (GSWindow  *window,
         gs_debug ("window unmapped!");
 }
 
+static void
+apply_background_to_window (GSManager *manager,
+                            GSWindow  *window)
+{
+        GdkPixmap       *pixmap;
+        GdkScreen       *screen;
+        int              width;
+        int              height;
+
+        if (manager->priv->bg == NULL) {
+                gs_debug ("No background available");
+                gs_window_set_background_pixmap (window, NULL);
+        }
+
+        screen = gs_window_get_screen (window);
+        width = gdk_screen_get_width (screen);
+        height = gdk_screen_get_height (screen);
+        gs_debug ("Creating pixmap background w:%d h:%d", width, height);
+        pixmap = gnome_bg_create_pixmap (manager->priv->bg,
+                                         gs_window_get_gdk_window (window),
+                                         width,
+                                         height,
+                                         TRUE);
+        gs_window_set_background_pixmap (window, pixmap);
+        g_object_unref (pixmap);
+}
+
 static void
 manager_show_window (GSManager *manager,
                      GSWindow  *window)
 {
         GSJob *job;
 
+        apply_background_to_window (manager, window);
+
         job = gs_job_new_for_widget (GTK_WIDGET (window));
 
         manager_select_theme_for_job (manager, job);
@@ -1517,6 +1646,7 @@ gs_manager_deactivate (GSManager *manager)
         }
 
         remove_unfade_idle (manager);
+        gs_fade_reset (manager->priv->fade);
         remove_timers (manager);
 
         gs_grab_release (manager->priv->grab);
